"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function getMetadata(source) {
    const imports = [];
    const solidity = [];
    const parser = new Parser(source);
    for (const item of parser.parse()) {
        if (item === undefined) {
            break;
        }
        if (item.kind === 'import') {
            imports.push(item.value);
        }
        if (item.kind === 'pragma' && item.name === 'solidity') {
            solidity.push(item.value);
        }
    }
    return { imports, solidity };
}
exports.getMetadata = getMetadata;
class Parser {
    constructor(source) {
        this.source = source;
        this.state = new StatefulRegExp(source);
    }
    *parse(semicolons = false) {
        while (true) {
            const item = this.next(semicolons);
            if (item === undefined) {
                break;
            }
            else {
                yield item;
            }
        }
    }
    next(semicolons) {
        var _a, _b, _c;
        const re = semicolons ? /import|pragma|\/\*|\/\/|"|'|;/ : /import|pragma|\/\*|\/\/|"|'/;
        const match = this.state.exec(re);
        if (match === null) {
            return undefined;
        }
        const anchor = match[0];
        const start = match.index;
        if (anchor === 'pragma') {
            const name = (_a = this.state.exec(/[a-zA-Z0-9$_]+/)) === null || _a === void 0 ? void 0 : _a[0];
            if (name !== undefined) {
                let value = '';
                let chunkStart = this.state.index;
                let end = -1;
                for (const item of this.parse(true)) {
                    if (item.kind === 'comment' || item.kind === 'semicolon') {
                        value += this.source.slice(chunkStart, item.start);
                        chunkStart = item.end;
                    }
                    if (item.kind === 'semicolon') {
                        end = item.start;
                        break;
                    }
                }
                return {
                    kind: 'pragma',
                    name,
                    value,
                    start,
                    end,
                };
            }
        }
        else if (anchor === 'import') {
            for (const item of this.parse()) {
                if (item.kind === 'string') {
                    return {
                        kind: 'import',
                        value: item.value,
                        end: item.end,
                        start,
                    };
                }
            }
        }
        else if (anchor === '"' || anchor === `'`) {
            const contentStart = this.state.index;
            this.state.exec(anchor);
            const contentEnd = this.state.index - 1;
            return {
                kind: 'string',
                value: this.source.slice(contentStart, contentEnd),
                start,
                end: this.state.index,
            };
        }
        else if (anchor === '/*') {
            const close = this.state.exec(/\*\//);
            return {
                kind: 'comment',
                start,
                end: this.state.index,
            };
        }
        else if (anchor === '//') {
            const close = this.state.exec(/\n/);
            return {
                kind: 'comment',
                start,
                end: (_c = (_b = close) === null || _b === void 0 ? void 0 : _b.index, (_c !== null && _c !== void 0 ? _c : this.state.index)),
            };
        }
        else if (anchor === ';') {
            return {
                kind: 'semicolon',
                start,
                end: this.state.index,
            };
        }
    }
}
class StatefulRegExp {
    constructor(source) {
        this.source = source;
        this.index = 0;
    }
    exec(re) {
        const sticky = new RegExp(re, 'g');
        sticky.lastIndex = this.index;
        const match = sticky.exec(this.source);
        if (match === null) {
            this.index = this.source.length;
        }
        else {
            this.index = sticky.lastIndex;
        }
        return match;
    }
}
//# sourceMappingURL=fuzzy-parser.js.map