"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ava_1 = __importDefault(require("ava"));
const _1 = require(".");
ava_1.default('parses empty file', t => {
    const { imports, solidity } = _1.getMetadata('');
    t.deepEqual(imports, []);
    t.deepEqual(solidity, []);
});
ava_1.default('gets import token', t => {
    const parser = new _1.Parser('import "./token.sol";');
    const token = parser.consume();
    t.is(token.kind, _1.TokenKind.Import);
    t.is(token.value, "./token.sol");
});
ava_1.default('gets pragma token', t => {
    const parser = new _1.Parser('pragma solidity ^0.5.0;');
    const token = parser.consume();
    t.is(token.kind, _1.TokenKind.Pragma);
    t.is(token.value, "^0.5.0");
});
ava_1.default('gets single import', t => {
    const parser = new _1.Parser('import "./token.sol";');
    const { imports } = parser.getMetadata();
    t.deepEqual(imports, ["./token.sol"]);
});
ava_1.default('gets two imports', t => {
    const parser = new _1.Parser('import "./token.sol"; import "./token2.sol";');
    const { imports } = parser.getMetadata();
    t.deepEqual(imports, ["./token.sol", "./token2.sol"]);
});
ava_1.default('gets two pragmas', t => {
    const parser = new _1.Parser('pragma solidity ^0.5.0; pragma solidity ^0.5.1;');
    const { solidity } = parser.getMetadata();
    t.deepEqual(solidity, ["^0.5.0", "^0.5.1"]);
});
ava_1.default('skips a contract', t => {
    const { imports } = _1.getMetadata(`
    contract Foo { }
    import "./token.sol";
  `);
    t.deepEqual(imports, ["./token.sol"]);
});
ava_1.default('skips a contract with functions', t => {
    const { imports } = _1.getMetadata(`
    contract Foo {
      function test() public returns (uint) {
      }
      function test() public returns (uint) {
      }
    }
    import "./token.sol";
  `);
    t.deepEqual(imports, ["./token.sol"]);
});
ava_1.default('skips a library and an interface', t => {
    const { imports } = _1.getMetadata(`
    library Foo {}
    interface Foo {}
    import "./token.sol";
  `);
    t.deepEqual(imports, ["./token.sol"]);
});
ava_1.default('skips line comment', t => {
    const { imports } = _1.getMetadata(`
    contract Foo {
      // docs
      function () {
      }
    }
    import "./token.sol";
  `);
    t.deepEqual(imports, ["./token.sol"]);
});
ava_1.default('skips comment', t => {
    const { imports } = _1.getMetadata(`
    contract Foo {
      /* docs */
      function () {
      }
    }
    import "./token.sol";
  `);
    t.deepEqual(imports, ["./token.sol"]);
});
ava_1.default('skips comments with opening brace', t => {
    const { imports } = _1.getMetadata(`
    contract Foo {
      /* { */
      // {
      function () {
      }
    }
    import "./token.sol";
  `);
    t.deepEqual(imports, ["./token.sol"]);
});
ava_1.default('skips string with opening brace', t => {
    const { imports } = _1.getMetadata(`
    contract Foo {
      function () {
        return "hehe }";
      }
    }
    import "./token.sol";
  `);
    t.deepEqual(imports, ["./token.sol"]);
});
ava_1.default('fails with unbalanced braces', t => {
    const { imports } = _1.getMetadata(`
    contract Foo } {
    import "./token.sol";
  `);
    t.deepEqual(imports, ["./token.sol"]);
});
ava_1.default('skips top level comments', t => {
    const { imports } = _1.getMetadata(`
    /* skip me */
    import "./token.sol";
  `);
    t.deepEqual(imports, ["./token.sol"]);
});
ava_1.default('ignores other pragmas', t => {
    const { solidity } = _1.getMetadata(`
    pragma abiEncoderV2 true;
    pragma solidity ^0.5.0;
  `);
    t.deepEqual(solidity, ["^0.5.0"]);
});
ava_1.default('skips a contract with inheritance', t => {
    const { imports } = _1.getMetadata(`
    contract Foo is Bar, Baz(1) {
    }
    import "./token.sol";
  `);
    t.deepEqual(imports, ["./token.sol"]);
});
ava_1.default('gets import with single quotes', t => {
    const { imports } = _1.getMetadata(`
    import './token.sol';
  `);
    t.deepEqual(imports, ["./token.sol"]);
});
ava_1.default('skips single qoute string with opening brace', t => {
    const { imports } = _1.getMetadata(`
    contract Foo {
      function () {
        return 'hehe }';
      }
    }
    import "./token.sol";
  `);
    t.deepEqual(imports, ["./token.sol"]);
});
ava_1.default('gets import with named imports', t => {
    const { imports } = _1.getMetadata(`
    import { x } from './token.sol';
  `);
    t.deepEqual(imports, ["./token.sol"]);
});
ava_1.default('skips inheritance with constructor argument struct', t => {
    const { imports } = _1.getMetadata(`
    contract Foo is Bar(Base.X({x:1})) {
      function () {
        return 'hehe }';
      }
    }
    import "./token.sol";
  `);
    t.deepEqual(imports, ["./token.sol"]);
});
ava_1.default('gets false positive import', t => {
    const { imports } = _1.getMetadata(`
    contract Foo {
      import "./token.sol";
    }
  `);
    t.deepEqual(imports, ["./token.sol"]);
});
ava_1.default('handles line comment not ending in newline', t => {
    const { imports } = _1.getMetadata(`import "./foo.sol"; // import "./token.sol";`);
    t.deepEqual(imports, ["./foo.sol"]);
});
ava_1.default('handles multiline comment not closed', t => {
    const { imports } = _1.getMetadata(`import "./foo.sol"; /* import "./token.sol";`);
    t.deepEqual(imports, ["./foo.sol"]);
});
ava_1.default('handles pragma statement not ending in semicolon', t => {
    const { solidity } = _1.getMetadata(`pragma solidity ^0.5.0`);
    t.deepEqual(solidity, ["^0.5.0"]);
});
//# sourceMappingURL=index.test.js.map